import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, Empty, notification, Input, Tree, Icon } from 'antd'
import moment from 'moment'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import UtilsDM from '@/utils/utils-datamanage.js'
import MKEmitter from '@/utils/events.js'
import './index.scss'

const { TreeNode } = Tree
const { Search } = Input

class NormalTree extends Component {
  static propTpyes = {
    BID: PropTypes.any,              // 父级Id
    data: PropTypes.array,           // 统一查询数据
    config: PropTypes.object,        // 组件配置信息
    mainSearch: PropTypes.any,       // 外层搜索条件
    menuType: PropTypes.any,         // 菜单类型
  }

  state = {
    BID: '',                   // 主表ID
    config: null,              // 图表配置信息
    loading: false,            // 数据加载状态
    sync: false,               // 是否统一请求数据
    data: null,                // 数据
    searchkey: null,           // 过滤条件
    treedata: null,            // 列表数据集
    treeNodes: null,           // 列表数据集
    expandedKeys: [],          // 展开的树节点
    selectedKeys: [],          // 选中的树节点
  }

  UNSAFE_componentWillMount () {
    const { config, data, initdata, BID } = this.props
    let _config = fromJS(config).toJS()
    let _data = null
    let _sync = config.setting.sync === 'true'

    if (config.setting.sync === 'true' && data) {
      _data = data[config.dataName] || []
      _sync = false
    } else if (config.setting.sync === 'true' && initdata) {
      _data = initdata || []
      _sync = false
    }

    this.setState({
      config: _config,
      data: _data,
      BID: BID || '',
      arr_field: _config.columns.map(col => col.field).join(','),
      sync: _sync
    }, () => {
      if (config.setting.sync !== 'true' && config.setting.onload === 'true') {
        this.loadData()
      } else if (config.setting.sync === 'true' && _data) {
        this.handleData()
      }
    })
  }

  /**
   * @description 图表数据更新，刷新内容
   */
  UNSAFE_componentWillReceiveProps (nextProps) {
    const { sync, config } = this.state

    if (sync && !is(fromJS(this.props.data), fromJS(nextProps.data))) {
      let _data = []
      if (nextProps.data && nextProps.data[config.dataName]) {
        _data = nextProps.data[config.dataName] || []
      }

      this.setState({sync: false, data: _data}, () => {
        this.handleData()
      })
    } else if (config.setting.syncRefresh && nextProps.mainSearch && !is(fromJS(this.props.mainSearch), fromJS(nextProps.mainSearch))) {
      this.setState({}, () => {
        this.loadData()
      })
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    this.handleTimer()
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    clearTimeout(this.timer)
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
  }

  handleTimer = () => {
    const { config } = this.state

    if (!config.timer) return

    const _change = { '15s': 15000, '30s': 30000, '1min': 60000, '5min': 300000, '10min': 600000, '15min': 900000, '30min': 1800000, '1hour': 3600000 }

    let timer = _change[config.timer]

    if (!timer) return

    let _param = {
      func: 's_get_timers_role',
      LText: `select '${window.GLOB.appkey || ''}','${config.uuid}'`,
      timer_type: config.timer,
      component_id: config.uuid
    }
    
    _param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')          // 时间戳
    _param.LText = Utils.formatOptions(_param.LText)                   // 关键字符替换，base64加密
    _param.secretkey = Utils.encrypt(_param.LText, _param.timestamp)   // md5密钥

    Api.getSystemConfig(_param).then(result => {
      if (!result.status) {
        notification.warning({
          top: 92,
          message: result.message,
          duration: 5
        })
        return
      } else if (result.run_type) {
        this.setState({timer})
        this.timer = setTimeout(() => {
          this.timerTask()
        }, timer)
      }
    })
  }

  timerTask = () => {
    const { timer } = this.state
    if (!timer) return
    
    this.loadData(true)
    
    this.timer = setTimeout(() => {
      this.timerTask()
    }, timer)
  }

  reloadData = (menuId) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    this.loadData()
  }

  resetParentParam = (MenuID, id) => {
    const { config } = this.state

    if (!config.setting.supModule || config.setting.supModule !== MenuID) return
    if (id !== this.state.BID) {
      this.setState({ BID: id }, () => {
        this.loadData()
      })
    }
  }

  /**
   * @description 数据加载
   */
  async loadData (hastimer) {
    const { mainSearch, menuType } = this.props
    const { config, arr_field, BID } = this.state

    if (config.setting.supModule && !BID) { // BID 不存在时，不做查询
      this.setState({
        data: []
      }, () => {
        this.handleData()
      })
      return
    }

    let searches = config.setting.useMSearch && mainSearch ? mainSearch : []

    let requireFields = searches.filter(item => item.required && item.value === '')
    if (requireFields.length > 0) {
      return
    }

    if (!hastimer) {
      this.setState({
        loading: true
      })
    }

    let _orderBy = config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, arr_field, searches, _orderBy, '', '', BID, menuType)

    let result = await Api.genericInterface(param)
    if (result.status) {
      this.setState({
        data: result.data,
        loading: false
      }, () => {
        this.handleData()
      })
    } else {
      this.setState({
        loading: false,
        timer: null
      })
      notification.error({
        top: 92,
        message: result.message,
        duration: 10
      })
    }
  }

  handleData = () => {
    const { data, searchkey, config } = this.state
    if (!data || data.length === 0) {
      this.setState({
        treedata: [],
        treeNodes: [],
      })
      return
    }
    let parentNodes = []
    let _options = []
    let logMap = new Map()

    data.forEach(item => {
      let pval = item[config.wrap.parentField]
      let val = item[config.wrap.valueField]

      if (!val || logMap.has(val)) return

      logMap.set(val, true)
      if (pval === config.wrap.mark) {
        parentNodes.push({
          // ...item,
          $title: item[config.wrap.labelField] || '',
          $key: val,
          $parentId: ''
        })
      } else if (pval) {
        _options.push({
          // ...item,
          $title: item[config.wrap.labelField] || '',
          $key: val,
          $parentId: pval
        })
      }
    })
    let _treedata = this.getTree(parentNodes, _options)
    
    let _treeNodes = []

    if (!searchkey) {
      _treeNodes = fromJS(_treedata).toJS()
    } else {
      _treeNodes = this.getFilterTree(fromJS(_treedata).toJS(), searchkey.toLowerCase())
    }

    this.setState({
      treedata: _treedata,
      treeNodes: _treeNodes,
    })
  }

  treeFilter = (value) => {
    const { treedata } = this.state

    let _treeNodes = []

    if (!value) {
      _treeNodes = fromJS(treedata).toJS()
    } else {
      _treeNodes = this.getFilterTree(fromJS(treedata).toJS(), value.toLowerCase())
    }

    this.setState({
      searchkey: value,
      treeNodes: _treeNodes
    })
  }

  /**
   * @description 获取结构树信息
   */
  getFilterTree = (parents, searchkey) => {
    return parents.filter(node => {
      if (!node.children) {
        return (node.$title.toLowerCase().indexOf(searchkey) >= 0 || node.$key.toLowerCase().indexOf(searchkey) >= 0)
      } else {
        if (node.$title.toLowerCase().indexOf(searchkey) >= 0 || node.$key.toLowerCase().indexOf(searchkey) >= 0) {
          return true
        }
        
        node.children = this.getFilterTree(node.children, searchkey)
        if (node.children.length === 0) {
          return false
        } else {
          return true
        }
      }
    })
  }

  /**
   * @description 获取结构树信息
   */
  getTree = (parents, options) => {
    parents.forEach(parent => {
      parent.children = []
      // 添加菜单的子元素
      options = options.filter(option => {
        if (option.$parentId === parent.$key) {
          parent.children.push(option)
          return false
        }
        return true
      })

      if (parent.children.length === 0) {
        parent.children = null
      } else {
        parent.children = this.getTree(parent.children, options)
      }
    })
    return parents
  }

  /**
   * @description 获取树节点
   */
  renderTreeNodes = (nodes) => {
    return nodes.map(item => {
      if (item.children) {
        return (
          <TreeNode icon={<span><Icon type="folder-open" /><Icon type="folder" /></span>} title={item.$title} key={item.$key} dataRef={item}>
            {this.renderTreeNodes(item.children)}
          </TreeNode>
        )
      }
      return <TreeNode icon={<Icon type="file" />} key={item.$key} title={item.$title} dataRef={item} isLeaf />
    })
  }

  changeExpandedKeys = (expandedKeys) => {
    this.setState({
      expandedKeys: expandedKeys
    })
  }

  // 右键展开节点下的全部分支
  changeExpandedAllKeys = ({event, node}) => {
    const { expandedKeys } = this.state
    let _node = node.props.dataRef
    event.stopPropagation()

    let keys = []
    this.getExpandKeys(_node, keys)

    this.setState({
      expandedKeys: Array.from(new Set([...keys, ...expandedKeys])),
    })
  }

  getExpandKeys = (node, keys) => {
    if (node.children) {
      keys.push(node.$key)
      node.children.forEach(_node => {
        this.getExpandKeys(_node, keys)
      })
    }
  }

  selectTreeNode = (selectedKeys, {selected, node}) => {
    const { config } = this.state
    let _expandedKeys = fromJS(this.state.expandedKeys).toJS()
    let _data = fromJS(node.props.dataRef).toJS()

    if (_expandedKeys.indexOf(_data.$key) >= 0) {
      _expandedKeys = _expandedKeys.filter(key => key !== _data.$key)
    } else {
      if (_data.children) {
        _expandedKeys.push(_data.$key)
        _expandedKeys = Array.from(new Set(_expandedKeys))
      }
    }

    if (selected) {
      MKEmitter.emit('resetSelectLine', config.uuid, (_data ? _data.$key : ''), _data)
    }

    this.setState({
      expandedKeys: _expandedKeys,
      selectedKeys: [_data.$key]
    })
  }

  render() {
    const { config, loading, treeNodes, expandedKeys, selectedKeys } = this.state

    return (
      <div className="custom-tree-box" style={config.style}>
        {loading ?
          <div className="loading-mask">
            <div className="ant-spin-blur"></div>
            <Spin />
          </div> : null
        }
        {config.wrap.title || config.wrap.searchable === 'true' ? <div className="tree-header" style={config.headerStyle}>
          <span className="title">{config.wrap.title}</span>
          {config.wrap.searchable === 'true' ? <Search allowClear onSearch={this.treeFilter} /> : null}
        </div> : null}
        {treeNodes && treeNodes.length > 0 ? <div className="tree-box">
          <Tree
            blockNode
            onSelect={this.selectTreeNode}
            expandedKeys={expandedKeys}
            selectedKeys={selectedKeys}
            onRightClick={this.changeExpandedAllKeys}
            onExpand={this.changeExpandedKeys}
            showIcon={config.wrap.showIcon === 'true'}
            showLine={config.wrap.showLine === 'true'}
          >
            {this.renderTreeNodes(treeNodes)}
          </Tree>
        </div> : null}
        {treeNodes && treeNodes.length === 0 ? <Empty description={false}/> : null}
      </div>
    )
  }
}

export default NormalTree